import subprocess
from subprocess import Popen
import dotenv 
import requests_unixsocket
import os
from ain.instanceUtil import InstanceUtil 
import ain.constants as constants


class Worker():

	def __init__(self):
		self.ENVS = dotenv.dotenv_values(constants.ENV_FILE_PATH)
		self.session = requests_unixsocket.Session()
		self.hostTtydSocket = f'{constants.SHARED_PATH}/ain_worker_ttyd.sock'
		sharedPath = constants.SHARED_PATH.replace("/", '%2F')
		self.workerAddress =  f'http+unix://{sharedPath}%2Fain_worker.sock'

	def run(self, optionRun):

		for key in optionRun:
			if self.ENVS[key].replace(" ", "") == "" and optionRun[key] == "":
				print(f'[-] {key} empty')
				return

			if optionRun[key] != "":
				dotenv.set_key(constants.ENV_FILE_PATH, key, optionRun[key])

		print("[?] Do you want to start? (y/n)")
		answer = input()
		if (answer.lower() != 'y'):
			return

		# open provider's ttys socket 
		InstanceUtil.createTtyd(self.hostTtydSocket)

		# open docker container for ain worker server
		InstanceUtil.createContainer(constants.IMAGE, self.ENVS)


	def status(self):
		try:
			response = self.session.get(self.workerAddress + "/info")
			
			ids = response.json()['id']

			if(len(ids) == 0):
				print("[+] does not exist")
				return
		except Exception as e:
			print("[-] worker server error")
			print(e)
			exit(1)
		
		try:
			option = " ".join(ids)
			subprocess.run(["docker", "stats" , option])

		except Exception as e:
			print("[-] subprocess(docker) error")
			print(e)

	def terminate(self):
		try:      
			response = self.session.get(self.workerAddress + "/info")
			cnt = response.json()['cnt']
			if (cnt != 0):
				print(f"[+] instance count: {cnt}")
				print("[+] (y/n)")
				answer = input()
				if (answer.lower() != 'y'):
					return
		except Exception as e:
			print("[-] worker server error - info")
			print(e)

		try:
			response = self.session.get(self.workerAddress + "/terminate")
		except Exception as e:
			print("[-] server error - terminate")
			print(e)
	
		try:
			InstanceUtil.removeContainer("ain_worker")
			print('[+] succeded to remove container!')
		
		except Exception as e:
			print("[-] subprocess(docker) error")
			print(e)

		try:
			InstanceUtil.removeTtyd(self.hostTtydSocket)
			InstanceUtil.removeTtyd(f'{constants.SHARED_PATH}/ain_worker.sock')
			print('[+] succeded to remove ttyd socket')
		except Exception as e:
			print("[-] subprocess error(ttyd socker remove)")
			print(e)
			exit(1)

	def log(self):
		basePath = f"{constants.SHARED_PATH}/log/"
		if (os.path.exists(basePath + "log.env")):
			os.remove(basePath + "log.env")
		
		logFileName = os.listdir(basePath)
		times = [i.split("_")[2].split(".")[0] for i in logFileName]
		if (len(times) == 0) :
			print("[+] does not exist")
		times.sort()             
		try:
			path = os.path.join(basePath, "ain_worker_" + times[-1] + ".log")
			subprocess.run(["tail", "-f" , path])
		except Exception as e:
			print("[-] subprocess(log) error")
			print(e)

	def init(self):
		for key in ["NAME", "LIMIT_COUNT", "PRICE", "DESCRIPTION", "MNEMONIC", "SERVER_IP", "GPU"]:
			dotenv.set_key(constants.ENV_FILE_PATH, key, "")


from setuptools import setup, find_packages

NAME = "ainainainain"
VERSION = "1.5"

packages = [
  "setuptools",
  "wheel",
  "python-dotenv==0.10.1",
  "docker==3.7.0",
  "requests-unixsocket==0.1.5",
  "click==7.0"
]

setup(
    package_data = {
        '': ['share/*.env','share/log/*.env'],
        'share/log': ['*.env'],
        'share': ['*.env'],
    },
    name = NAME,
    version = VERSION,
    python_requires='>=3.6',
    description = f"CLI of {NAME} a Python script",
    license = "BSD",
    url = "https://bitbucket.org/comcomai/ain-worker",
    packages=find_packages(),
    install_requires=packages,
    entry_points = {
        'console_scripts' : [f'ain = ain.ain:call']
    },
    classifiers=[
        "License :: OSI Approved :: BSD License"
    ]
)



import subprocess
from subprocess import Popen
import socket
import docker
import os
import ain.constants as constants


class InstanceUtil():

  @staticmethod
  def createTtyd(socketName):
    try:
      print("[+] open ttyd socket.")
      Popen(["ttyd" ,"-i" ,socketName ,"/bin/bash"],
                shell=False, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    except Exception as e:
      print(f"[-] failed to create ttyd socket. - {socketName}")
      print(e)
      exit(1)

  @staticmethod
  def removeTtyd(socketName):
    try:
      if (os.path.exists(socketName)):
        os.remove(socketName)
    except Exception as e:
      print(f"[-] failed to remove ttyd socket. - {socketName}")
      print(e)
      exit(1)

  @staticmethod
  def createContainer(image, envs):
    try:
      client = docker.from_env()
      client.containers.run(image=image, command="yarn start",
                            detach=True,
                            name="ain_worker",
                            volumes={
                              f'/var/run/docker.sock': {'bind': f'/var/run/docker.sock', 'mode': 'ro'},
                              constants.SHARED_PATH:{'bind': "/share", 'mode': "rw"}
                              }
                            )
      
      print("[+] started ain worker server!")
    except Exception as e:
      print("[-] failed to create container.")
      print(e)
      exit(1)

  @staticmethod
  def removeContainer(name):
    try:
      client = docker.from_env()
      constainer = client.containers.get(name)
      constainer.remove(force=True)
      
    except Exception as e:
      print("failed to remove container.")
      print(e)


import click
from ain.workerMgr import Worker

@click.group()
@click.option('--type/--no-type', default=False)
def call(type):
    pass

@call.command()
@click.argument("command", type=click.Choice(['run', 'terminate', 'status', 'log', 'init']))
@click.option('--name', default="", help="ain worker name")
@click.option('--limit-count', default="", help="Maximum number of instances")
@click.option('--price', default="")
@click.option('--mnemonic', default="")
@click.option('--description', default="empty")
@click.option('--server-ip', default="empty")
@click.option('--gpu', default="false")
def worker(command, name, limit_count, price, mnemonic, description, server_ip, gpu):
    
    optionRun = {
      'NAME': name,
      'LIMIT_COUNT': limit_count,
      'MNEMONIC': mnemonic,
      'PRICE': price,
      'DESCRIPTION': description,
      'GPU': gpu,
      'SERVER_IP': server_ip
    }

    w = Worker()  
    if (command == "run"):
      w.run(optionRun)
    elif (command == "terminate"):
      w.terminate()
    elif (command == "status"):
      w.status()
    elif (command == "log"):
      w.log()
    elif (command == "init"):
      w.init()

    
if __name__ == '__main__':
    call()
